# -*- coding: utf-8 -*-
from django.urls import reverse
from django.utils import timezone
from django.conf import settings
from django.test import tag

from guichet.tests import CustomTestCase

import pytz

from accounts.models import User
from guichet.models import PaypalAccount, Event, Registration
from paypal.standard.ipn.models import PayPalIPN
from paypal.standard.models import ST_PP_COMPLETED, ST_PP_FAILED


# deux modèles pour simuler les réponses de paypal lors de la vérification des IPN
class PayPalIPNValidTest(PayPalIPN):
    def _postback(self):
        return b"VERIFIED"

    class Meta:
        proxy = True


class PayPalIPNNotValidTest(PayPalIPN):
    def _postback(self):
        return b"UNVERIFIED"

    class Meta:
        proxy = True


class PaypalTest(CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.tz = pytz.timezone("Europe/Paris")
        self.now = timezone.now().astimezone(self.tz)
        self.paypalaccount = PaypalAccount.objects.get(email="test@test.fr")
        self.event = Event.objects.get(pk=1)
        self.event.paypal_account = self.paypalaccount
        self.event.save()
        self.ticket = self.event.tickets.get(id=1)
        self.ticket.online_payment = True
        self.ticket.save()
        self.user = User.objects.get(login_inp_net="testwo")
        self.reg = Registration(ticket=self.ticket, owner=self.user, conditions_accepted=True)
        self.reg.save()
        self.ipnobj = {
            "business": self.paypalaccount.email,
            "charset": "windows-1252",
            "custom": "",
            "notify_version": "3.90",
            "parent_txn_id": "",
            "receiver_email": self.paypalaccount.email,
            "receiver_id": "m1iGFQIk7PyCU",
            "residence_country": "FR",
            "test_ipn": 0,
            "txn_id": "30CM6xbqST2GiJtVY",
            "txn_type": "web_accept",
            "verify_sign": "q3ECrgcoQO2NvIIhgJQIToCUTpEQYTNFM6tYIwTTsSZEwAmp1vcJJRoL",
            "address_country": "",
            "address_city": "",
            "address_country_code": "",
            "address_name": "",
            "address_state": "",
            "address_status": "",
            "address_street": "",
            "address_zip": "",
            "contact_phone": "",
            "first_name": "Test",
            "last_name": "Test",
            "payer_business_name": "",
            "payer_email": self.user.email,
            "payer_id": "tSt8d3Ta9VWFF",
            "auth_amount": 0,
            "auth_exp": "",
            "auth_id": "",
            "auth_status": "",
            "exchange_rate": 0.0,
            "invoice": settings.PAYPAL_INVOICE_PREFIX + str(self.reg.pk),
            "item_name": self.event.name + " - " + self.ticket.name,
            "item_number": "",
            "mc_currency": "EUR",
            "mc_fee": float(self.ticket.price) * 0.2,
            "mc_gross": self.ticket.price,
            "mc_handling": 0.00,
            "mc_shipping": 0.00,
            "memo": "",
            "num_cart_items": 0,
            "option_name1": "",
            "option_name2": "",
            "payer_status": "unverified",
            "payment_date": self.now,
            "payment_gross": None,
            "payment_status": ST_PP_COMPLETED,
            "payment_type": "instant",
            "pending_reason": "",
            "protection_eligibility": "Eligible",
            "quantity": "1",
            "reason_code": "",
            "remaining_settle": 0.00,
            "settle_amount": 0.00,
            "settle_currency": "",
            "shipping": 0.00,
            "shipping_method": "Default",
            "tax": 0.00,
            "transaction_entity": "",
            "auction_buyer_id": "",
            "auction_closing_date": None,
            "auction_multi_item": 0,
            "for_auction": 0.00,
            "amount": 0.00,
            "amount_per_cycle": 0.00,
            "initial_payment_amount": 0.00,
            "next_payment_date": None,
            "outstanding_balance": 0.00,
            "payment_cycle": "",
            "period_type": "",
            "product_name": "",
            "product_type": "",
            "profile_status": "",
            "recurring_payment_id": "",
            "rp_invoice_id": "",
            "time_created": None,
            "amount1": 0.00,
            "amount2": 0.00,
            "amount3": 0.00,
            "mc_amount1": 0.00,
            "mc_amount2": 0.00,
            "mc_amount3": 0.00,
            "password": "",
            "period1": "",
            "period2": "",
            "period3": "",
            "reattempt": "",
            "recur_times": 0,
            "recurring": "",
            "retry_at": None,
            "subscr_date": None,
            "subscr_effective": None,
            "subscr_id": "",
            "username": "",
            "case_creation_date": None,
            "case_id": "",
            "case_type": "",
            "receipt_id": "",
            "currency_code": "USD",
            "handling_amount": 0.00,
            "transaction_subject": "",
            "ipaddress": "0.0.0.0",
            "flag": False,
            "flag_code": "",
            "flag_info": "",
            "query": "",  # supprimer car inutile
            "response": "",
            "created_at": self.now,
            "updated_at": self.now,
            "from_view": None,
            "mp_id": None,
            "option_selection1": "",
            "option_selection2": "",
        }

    def verify_and_send(self, ipn):
        ipn.save()
        ipn.verify()
        ipn.save()
        ipn.send_signals()

    @tag("fast", "local")
    def test_validIPN(self):
        ipn = PayPalIPNValidTest(**self.ipnobj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertFalse(reg.opposed)
        # get edit-registration to show IPN
        self.client.force_login(User.objects.get(login_inp_net="testone"))
        response = self.client.get(reverse("registration-edit", args=[self.event.short_url, self.reg.pk]))
        self.assertEqual(response.status_code, 200)

    @tag("fast", "nologin", "local")
    def test_invalideIPN(self):
        ipn = PayPalIPNNotValidTest(**self.ipnobj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertFalse(reg.paid)

    @tag("fast", "nologin", "local")
    def test_valideIPNBadReceiver(self):
        obj = self.ipnobj.copy()
        obj["business"] = "otheraccount@test.fr"
        obj["receiver_email"] = "otheraccount@test.fr"
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertTrue(reg.opposed)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)

    @tag("fast", "nologin", "local")
    def test_valideIPNBadCurrency(self):
        obj = self.ipnobj.copy()
        obj["mc_currency"] = "USD"
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertTrue(reg.opposed)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)

    @tag("fast", "nologin", "local")
    def test_valideIPNBadAmount(self):
        obj = self.ipnobj.copy()
        obj["mc_gross"] = 1
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertTrue(reg.opposed)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)

    @tag("fast", "nologin", "local")
    def test_valideIPNSandbox(self):
        obj = self.ipnobj.copy()
        obj["test_ipn"] = 1
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertTrue(reg.opposed)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)

    @tag("fast", "nologin", "local")
    def test_valideIPNBadInvoice(self):
        obj = self.ipnobj.copy()
        obj["invoice"] = "amazon-chantilly"
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertFalse(reg.paid)
        settings.PAYPAL_INVOICE_PREFIX + str(self.reg.pk)

    @tag("fast", "nologin", "local")
    def test_valideIPNBadInvoice2(self):
        obj = self.ipnobj.copy()
        obj["invoice"] = settings.PAYPAL_INVOICE_PREFIX + "132354681897"
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertFalse(reg.paid)

    @tag("fast", "nologin", "local")
    def test_valideIPNBadStatus(self):
        obj = self.ipnobj.copy()
        obj["payment_status"] = ST_PP_FAILED
        ipn = PayPalIPNValidTest(**obj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertFalse(reg.paid)

    @tag("fast", "nologin", "local")
    def test_valideINPpayaplDisableTicket(self):
        self.ticket.online_payment = False
        self.ticket.save()
        ipn = PayPalIPNValidTest(**self.ipnobj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertTrue(reg.opposed)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)

    @tag("fast", "nologin", "local")
    def test_valideINPpayaplDisableEvent(self):
        self.event.paypal_account = PaypalAccount.default_paypal_account()
        self.event.save()
        ipn = PayPalIPNValidTest(**self.ipnobj)
        self.verify_and_send(ipn)
        reg = Registration.objects.get(pk=self.reg.pk)
        self.assertTrue(reg.paid)
        self.assertTrue(reg.opposed)
        self.assertEqual(reg.reason_opposed, Registration.OPPOSED_BY_PAYPAL)
